package pg;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

@SuppressWarnings("serial")
class Bezier extends Canvas {

	public Vector<Ponto> pontos;
	public int active;
	private Dimension tamanhoTela;
	public MouseListener mouseListener;
	public MouseMotionListener mouseMotionListener;
	public boolean pontosEscondidos;
	public boolean poligonoApagado;
	public boolean curvaApagada;
	private JanelaCurvatura janelaDerivada;
	private int qtdDeAvaliacoes;

	public Bezier(int largura, int altura) {

		this.poligonoApagado = false;
		this.pontosEscondidos = false;
		this.curvaApagada = false;
		tamanhoTela = new Dimension(largura, altura);
		pontos = new Vector<Ponto>();
		pontos.add(new Ponto(20, 120));
		pontos.add(new Ponto(70, 30));
		pontos.add(new Ponto(130, 20));
		pontos.add(new Ponto(120, 130));
		active = -1;
		setBackground(Color.BLACK);
		qtdDeAvaliacoes = 4;
		janelaDerivada = new JanelaCurvatura(this.pontos);

		// adicionando acao do click do mouse
		mouseListener = new MouseListener() {

			@Override//click do mouse para adicionar um ponto
			public void mouseClicked(MouseEvent e) {
				Ponto novoPontoControle = new Ponto(e.getX(), e.getY());
				pontos.add(novoPontoControle);
				repaint();
				setJanelaDerivada();
			}

			//metodos inuteis
			@Override
			public void mouseEntered(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseExited(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mousePressed(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseReleased(MouseEvent e) {
				// TODO Auto-generated method stub

			}

		};
		this.addMouseListener(mouseListener);

		// adicionando acoes de movimento do mouse
		mouseMotionListener = new MouseMotionListener() {

			
			
			@Override//movimento do mouse para editar posio do ponto
			public void mouseDragged(MouseEvent e) {
				
				if (active == -1) {
					return;
				}
				if (outside(e.getX(), e.getY())) {
					return;
				}
				apagaGraficoAnterior();
				pontos.get(active).setX(e.getX());
				pontos.get(active).setY(e.getY());
				desenharGrafico();
				setJanelaDerivada();
			}

			@Override
			public void mouseMoved(MouseEvent e) {
				for (int i = 0; i < pontos.size(); ++i) {
					if (pontos.get(i).near(e.getX(), e.getY())) {
						active = i;
						return;
					}
				}
				active = -1;
				
			}

		};
		this.addMouseMotionListener(mouseMotionListener);
	}

	public void setJanelaDerivada(){
		janelaDerivada.setVetorPontos(pontos);
	}
	
	public void setQtdAvaliacoes(int qtdAvaliacoes) {

		apagaGraficoAnterior();
		qtdDeAvaliacoes = qtdAvaliacoes;
		desenharGrafico();
	}

	public void desenharGrafico() {

		Graphics g = getGraphics();
		if(!this.poligonoApagado){
			Grafico.desenhaPoligono(g, pontos, Color.GREEN);			
		}
		if(!this.curvaApagada){
			Grafico.calculaCurva(g, pontos, Color.RED, qtdDeAvaliacoes);			
		}
		if(!this.pontosEscondidos){	
			for (int i = 0; i < pontos.size(); ++i) {
				Grafico.desenhaPonto(g, pontos.get(i), Color.WHITE);
				g.setColor(Color.WHITE);
				g.drawString(("p"+ i), (int)pontos.get(i).x, (int)pontos.get(i).y);
			}
		}
	}

	public void apagaGraficoAnterior() {

		Graphics g = getGraphics();
		Grafico.desenhaPoligono(g, pontos, getBackground());
		Grafico.calculaCurva(g, pontos, getBackground(), qtdDeAvaliacoes);
		for (int i = 0; i < pontos.size(); ++i) {
			Grafico.desenhaPonto(g, pontos.get(i), getBackground());
			g.setColor(getBackground());
			g.drawString(("p"+ i), (int)pontos.get(i).x, (int)pontos.get(i).y);
		}
	}

	public void paint(Graphics g) {
		if (pontos.size() >= 2) {
			desenharGrafico();
		}
	}

	public boolean mouseDown(Event event, int x, int y) {

		for (int i = 0; i < pontos.size(); ++i) {
			if (pontos.get(i).near(x, y)) {
				active = i;
				return true;
			}
		}
		active = -1;
		return false;
	}

	private boolean outside(int x, int y) {
		if (x < 0)
			return true;
		if (y < 0)
			return true;
		if (x > tamanhoTela.width)
			return true;
		if (y > tamanhoTela.height)
			return true;
		return false;
	}

	public boolean mouseDrag(Event event, int x, int y) {
		if (active == -1) {
			return false;
		}
		if (outside(x, y)) {
			return true;
		}
		apagaGraficoAnterior();
		pontos.get(active).setX(x);
		pontos.get(active).setY(y);
		desenharGrafico();
		return true;
	}

	public void limparTela() {
		pontos.removeAllElements();
		this.repaint();
	}

	public void removerPonto() {
		pontos.remove(pontos.size() - 1);
		this.repaint();
	}
	
	public void removerPoligono() {
		this.poligonoApagado = !this.poligonoApagado;
		this.repaint();
	}
	
	public void suprimirPontos() {
		this.pontosEscondidos = !this.pontosEscondidos;
		this.repaint();		
	}
	
	
	public void suprimirCurva() {
		this.curvaApagada = !this.curvaApagada;
		this.repaint();	
	}
	
	public Vector<Ponto> getPontos(){
		return this.pontos;
	}
		 
}